/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_player_mp3http_h
#define GUARD_player_mp3http_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_mp3http.h
/// @brief Class for play mp3 on INET header file.

#include <unistd.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include <vector>
#include <string>

#include <glibmm.h>

#include <mad.h>

#include <ace/INET_Addr.h>
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>

#include "player.h"
#include "player_mp3.h"
#include "connector.h"

class MainManager;

namespace MAD {

    struct StreamBuffer {

        unsigned char* data;
        unsigned long length;

    };

    /// @brief Class for play mp3 on HTTP.
    ///
    /// MAD requires static function, almost member functions and variables are static.
    class MP3HttpPlayer : public Player 
    {
    public:
        MP3HttpPlayer(MainManager& mainManager);
        virtual ~MP3HttpPlayer();

        virtual bool Init(const std::string& destUrl);

        virtual void Start();

        virtual void Stop(); 
	    
        virtual void Seek(const unsigned long& seconds);

    private:
        MP3HttpPlayer& operator=(const MP3HttpPlayer& mp3HttpPlayer);
        MP3HttpPlayer(const MP3HttpPlayer& mp3HttpPlayer);
	
        static MainManager* mainManager_;		///< Program control MainManager
	    
        Glib::Thread* recvLoopThread_;
        static Glib::Mutex* recvLoopMutex_;
        static Glib::Cond* recvLoopCondPush_;
        static Glib::Cond* recvLoopCondPop_;

        Glib::Dispatcher signalFinish_;		///< Dispatcher for finish signal
        static Glib::Dispatcher signalUpdateStatusbar_;
        static Glib::Dispatcher signalUpdateStatusbarTimeoutClear_;
        static Glib::Dispatcher signalClearStatusbar_;
        static std::vector<double> bufSample_;	///< Container to store decoded samples

        struct mad_decoder decoder_;
        struct StreamBuffer buf_;
	    
        static DecodeState decodeState_;
        static unsigned int sampleRate_;
        static unsigned int recvBufSize_;

        static bool inRecving_;
        static bool disconnected_;

        static const unsigned int recvBufMaxSize_ = 131072;
        static const unsigned int recvBufLimitSize_ = 98304;
        static const unsigned int inputBufSize_ = 32768;

        static char recvBuf_[recvBufMaxSize_];
        static char inputBuf_[inputBufSize_];

        static ACE_INET_Addr destAddress_;
        static ACE_SOCK_Connector connector_;
        static ACE_SOCK_Stream peer_;

        std::string destUrl_;

        void Clean();
	    
        void Reset();

        void RecvLoop();

        //bool GetHeader();
        NetUtil::HTTPHeaderStatus GetHeader();

        void AllocateThreadVariables()
        {
            recvLoopMutex_ = new Glib::Mutex;
            recvLoopCondPush_ = new Glib::Cond;
            recvLoopCondPop_ = new Glib::Cond;
        }

        void DeallocateThreadVariables()
        {
            if(recvLoopCondPop_ != NULL) {
                delete recvLoopCondPop_;
                recvLoopCondPop_ = NULL;
            }

            if(recvLoopCondPush_ != NULL) {
                delete recvLoopCondPush_;
                recvLoopCondPush_ = NULL;
            }

            if(recvLoopMutex_ != NULL) {
                delete recvLoopMutex_;
                recvLoopMutex_ = NULL;
            }
        }

        static enum mad_flow Input(void* data, struct mad_stream* stream);
        static enum mad_flow Output(void* data, struct mad_header const* header, struct mad_pcm* pcm);
        static enum mad_flow Header(void* data, struct mad_header const* header);
        static enum mad_flow Error(void* data, struct mad_stream* stream, struct mad_frame* frame); 
    };

}

std::auto_ptr<Player> CreateMP3HttpPlayer(MainManager& mainManager);

#endif
