/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_player_wma_h
#define GUARD_player_wma_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// wma decoder came from bmp-wma plugin.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file player_wma.h
/// @brief Class for play wma header file.

#include <vector>
#include <string>

#include <glibmm.h>

#include "player.h"

#include "../src/libffwma/avcodec.h"
#include "../src/libffwma/avformat.h"

class MainManager;

namespace FFMPEG {

    /// @brief Class for play wma.
    class WMAPlayer : public Player
    {
    public:
        WMAPlayer(MainManager& mainManager);
        virtual ~WMAPlayer();
	    
        virtual bool Init(const std::string& filename);
        virtual void Start();
        virtual void Stop();
        virtual void Seek(const unsigned long& seconds);

    private:
        WMAPlayer& operator=(const WMAPlayer& wmaPlayer);
        WMAPlayer(const WMAPlayer& wmaPlayer);

        Glib::Dispatcher signalFinish_;	    ///< Dispatcher for finish signal.

        Glib::Mutex avMutex_;		    ///< Mutex to protect decodeState.

        DecodeState decodeState_;		    ///< DecodeState to check status.

        unsigned long int seekSeconds_;	    ///< Wanted seek time in seconds.
        const unsigned int bufSize_;			    ///< Buffer size.
        const unsigned int count_;
        int sampleRate_;			    ///< Current sample rate.
        int index_;	
	    
        bool validFile_;			    ///< Success of open file.

        std::vector<double> buf_;		    ///< Container to store decoded samples.

        MainManager& mainManager_;		    ///< Program control MainManager.

        AVCodec* avCodec_;
        AVPacket avPacket_;
        AVCodecContext* avCodecContext_;
        AVFormatContext* avFormatContext_;

        uint8_t* outbuf_;
        
        /// @brief Clean up decoder.
        void Clean()
        {
            if(outbuf_)
                free(outbuf_);

            if(validFile_) {
                avcodec_close(avCodecContext_);
                av_close_input_file(avFormatContext_);
            }
        }

        void ShortIntSignalToDoubleSignal(short int* const orig, unsigned int& origIndex, unsigned int& destIndex)
        {
            buf_[destIndex] = orig[origIndex];
            buf_[destIndex] += 0.5;
            buf_[destIndex] /= 32767.5;
            ++origIndex;
            ++destIndex;
        }
        void NullSignal(unsigned int& origIndex, unsigned int& destIndex)
        {
            buf_[destIndex] = 0.0;
            ++origIndex;
            ++destIndex;
        }
        void ProcessStereo(short int* const tempSample, unsigned int& outbufSize, unsigned int& bufIndex, unsigned int& index);
        void ProcessMono(short int* const tempSample, unsigned int& outbufSize, unsigned int& bufIndex, unsigned int& index);
        void Play(const int& bufferSize, const unsigned int& channels);

    };
}

std::auto_ptr<Player> CreateWMAPlayer(MainManager& mainManager);

#endif
