#ifndef GUARD_sockethandler_h
#define GUARD_sockethandler_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file sockethandler.h
/// @brief Class for unix socket handler header file.

#include <vector>
#include <string>

#include <glibmm.h>

#include "socket.h"

class MainManager;

/// @brief Class for unix socket handler.
class UnixSocketHandler
{
    public:
	UnixSocketHandler(const std::vector<std::string>& arguments);
	~UnixSocketHandler();

	bool Connect()
	{
	    return unixSocket_.Connect();
	}
	void SendArguments();

	/// @brief Start server loop.
	///
	/// If success on listen, execute acecpt loop thread.
	void StartServerLoop()
	{
	    if(unixSocket_.Listen()) {
		acceptThread_ = Glib::Thread::create(sigc::mem_fun(*this, &UnixSocketHandler::AcceptLoop), true);
		inThread_ = true;
	    }
	}

	/// @brief Copy received data to dest and empty received data container.
	/// @param dest Destination to copy received data
	void ReceivedData(std::vector<std::string>& dest)
	{
	    Glib::Mutex::Lock lock(acceptMutex_);

	    dest = receivedData_;

	    std::vector<std::string>().swap(receivedData_);
	}

	void SetMainManager(MainManager* const mainManager);

    private:
	UnixSocketHandler(const UnixSocketHandler& unixSocketHandler);
	UnixSocketHandler& operator=(const UnixSocketHandler& unixSocketHandler);

	const std::vector<std::string>& arguments_; ///< Command line arguments.
	std::vector<std::string> receivedData_;	    ///< A container to store received data.

	UnixSocket unixSocket_;

	Glib::Mutex acceptMutex_;		    ///< A mutex variable to protect container.
	Glib::Dispatcher signalReceived_;	    ///< A dispatcher variable for signal to MainManager.
	const unsigned int mainHeaderSize_;
	bool inThread_;
	bool stopThread_;
	Glib::Thread* acceptThread_;

	MainManager* mainManager_;

	void GetTotalBytesToSend(unsigned int& totalBytes) const
	{
	    for(std::vector<std::string>::const_iterator iter = arguments_.begin();
		iter != arguments_.end(); ++iter) {
		
		totalBytes += iter->size();
		totalBytes += UnixSocket::GetIntegerSize();

	    }
	}
	void SendHeader(const unsigned int& totalBytes);
	void SendData(const unsigned int& totalBytes);

	void RecvEntireData(const int& fd, char* const buf, const int& bufSize, const unsigned int& receivedBytes, unsigned int& totalBytes, char* const rcvData);
	void ExtractReceivedData(const char* const rcvData, const unsigned int& totalBytes);

	void AcceptLoop();
	void DataReceived(const int& fd, const unsigned int& receivedBytes, char* const buf, const int& bufSize);
};

#endif

