/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_tag_aac_h
#define GUARD_tag_aac_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag_aac.h
/// @brief Class for mpeg 4 audio tag processing header file.

#include <cstdio>
#include <string>
#include <map>

#include "tag.h"
#include "player_aac.h"

class MainManager;

/// @brief Class for mpeg 4 audio tag processing header file.
class AACTag : public Tag
{
public:
    AACTag(const std::string& filename, const MainManager* const mainManager);
    virtual ~AACTag();

private:
    const std::string& filename_;

    bool process_;
    bool mp4File_;

    FILE* inFile_;

    const MainManager* const mainManager_;

    void ProcessAAC();

    void ProcessAll();
    virtual void ProcessYear();
    virtual void ProcessTrack();
    virtual void ProcessTitle();
    virtual void ProcessAlbum();
    virtual void ProcessGenre();
    virtual void ProcessArtist();
    virtual void ProcessComment();
    virtual void ProcessTotalTime();

    /// @brief Class for MP4 type file.
    class MP4
    {
    public:
        MP4(const std::string& filename, FILE* const inFile, const MainManager* const mainManager);
        ~MP4();
		
        void GetTotalTimeSeconds(unsigned long& totalTimeSeconds);
		
        void GetTitle(Glib::ustring& title);
        void GetArtist(Glib::ustring& artist);
        void GetAlbum(Glib::ustring& album);
        void GetTrack(Glib::ustring& track);
        void GetDate(Glib::ustring& date);
        void GetGenre(Glib::ustring& genre);
        void GetComment(Glib::ustring& comment);

    private:
        MP4(const MP4& mp4);
        MP4& operator=(const MP4& mp4);

        const std::string& filename_;
		
        int track_;
        unsigned int frameSize_;

        bool valid_;

        faacDecHandle decoder_;
        mp4ff_t* mp4ffFile_;
        mp4AudioSpecificConfig mp4ASC_;

        const std::auto_ptr<mp4ff_callback_t> mp4Cb_;

        const MainManager* const mainManager_;
        
        typedef int (*Mp4ffMetaFunction)(const mp4ff_t*, char**);
        typedef std::map<std::string, Mp4ffMetaFunction> Mp4ffMetaCallbacks;
        Mp4ffMetaCallbacks mp4ffMetaCallbacks_;

        bool Init();
        void GetMetaData(const std::string& key, Glib::ustring& output);
    };
};

std::auto_ptr<Tag> CreateAACTag(const std::string& filename, const MainManager* const mainManager);

#endif
