/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_tag_mp3_h
#define GUARD_tag_mp3_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag_mp3.h
/// @brief Class for process mp3 tag header file.

#include "tag.h"
#include <id3/tag.h>

class MainManager;

/// @brief Class for process mp3 tag.
class MP3Tag : public Tag {
public:
    MP3Tag(const std::string& filename, const MainManager* const mainManager);
    virtual ~MP3Tag();

private:
    enum FileType {
        CBR = 0,		///< Constant bitrate file.
        VBR = -1,		///< Variable bitrate file.
        XING_VBR = -2		///< Variable bitrate file with XING header.
    };

    char tmpString_[4096];
	
    const long xingFlagFrames_;
    long xingFlags_;
    unsigned long xingFrames_;

    ID3_Tag id3Tag_;
    ID3_Frame* id3Frame_;
    ID3_Field* id3Field_;

    const std::string& filename_;
    const MainManager* const mainManager_;

    virtual void ProcessYear();
    virtual void ProcessTrack();
    virtual void ProcessTitle();
    virtual void ProcessAlbum();
    virtual void ProcessGenre();
    virtual void ProcessArtist();
    virtual void ProcessComment();
    virtual void ProcessTotalTime();

    void ProcessTextField(Glib::ustring& dest, const ID3_FrameID& frameID, const bool& convertUTF8 = true, const Glib::ustring& failString = "");

    bool XingParse(struct mad_bitptr* const ptr, unsigned int bitlen);
};

std::auto_ptr<Tag> CreateId3Tag(const std::string& filename, const MainManager* const mainManager);

#endif
