/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
#ifndef GUARD_tag_ogg_h
#define GUARD_tag_ogg_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file tag_ogg.h
/// @brief Class for process ogg tag header file.

#include <string>
#include <map>

#include <cstdio>

#include <vorbis/vorbisfile.h>
#include <glibmm.h>

#include "tag.h"
#include "misc.h"

class MainManager;

/// @brief Class for process ogg tag.
///
/// Possible OGG Tag
///
/// single :
///
/// ALBUM, ARTIST, PUBLISHER, COPYRIGHT, DISCNUMBER, ISRC, EAN/UPN
/// LABEL, LABELNO, LICENSE, OPUS, SOURCEMEDIA, TITLE, TRACKNUMBER
/// VERSION, ENCODED-BY, ENCODING
///
/// multiple :
/// 
/// COMPOSER, ARRANGER, LYRICIST, AUTHOR, CONDUCTOR, PERFORMER
/// ENSEMBLE, PART, PARTNUMBER, GENRE, DATE, LOCATION, COMMENT
class OGGTag : public Tag {
public:
    OGGTag(const std::string& filename, const MainManager* const mainManager);
    virtual ~OGGTag();

private:
    std::multimap<std::string, std::string> tag_;
    const std::string& filename_;

    OggVorbis_File vorbisFile_;

    bool validFile_;

    FILE* inFile_;	

    const MainManager* const mainManager_;

    /// @brief Find field in multimap and assign to output.
    /// @param field Key name to find
    /// @param output Output.
    /// @param sub If cannot find field in multimap, assign sub to output.
    void FindAndAssign(const std::string& field, Glib::ustring& output, const std::string& sub = "")
    {
        // for Multiple exist tag
        typedef std::multimap<std::string, std::string>::iterator mapIter;
        typedef std::pair<mapIter, mapIter> mapIterPair;
        mapIterPair p = tag_.equal_range(field);

        if(p.first == p.second) {
            output = sub;

            return;
        }
	     
        for(; p.first != p.second; ++p.first) {
            output += p.first->second;
            output += ' ';
        }

        // vorbis use UTF-8 encoding default. Do i need converting to UTF-8 again to confirm it?
    }

    virtual void ProcessYear();
    virtual void ProcessTrack();
    virtual void ProcessTitle();
    virtual void ProcessAlbum();
    virtual void ProcessGenre();
    virtual void ProcessArtist();
    virtual void ProcessComment();
    virtual void ProcessTotalTime();
		
};

std::auto_ptr<Tag> CreateOGGTag(const std::string& filename, const MainManager* const mainManager);

#endif
