/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file dirplaywin.cc
/// @brief Class for playlist with directory browser source file.

#include <string>
#include <vector>

#include <gtkmm/scrolledwindow.h>

#include "../include/dirplaywin.h"

#include "../include/mainwin.h"
#include "../include/player.h"
#include "../include/playlist.h"
#include "../include/dirbrowsewin.h"
#include "../include/misc.h"

/// @brief Class for implementation of DirPlayWnd.
class DirPlayWnd::DirPlayWndImpl
{
public:
    DirPlayWndImpl(MainWnd& mainWnd, DirPlayWnd& dirPlayWnd, const unsigned int& titleWidth);
    ~DirPlayWndImpl() {};

    DirBrowseWnd dirBrowseWnd_;
    Gtk::ScrolledWindow scrolledWindow_;
    Playlist playlist_;
    MainWnd& mainWnd_;

    void OnSelected();
};

/// @brief Constructor of DirPlayWnd.
/// @param mainWnd MainWnd
/// @param titleWidth Width of title column.
DirPlayWnd::DirPlayWnd(MainWnd& mainWnd, const unsigned int& titleWidth)
    : impl_(new DirPlayWndImpl(mainWnd, *this, titleWidth))
{}

DirPlayWnd::~DirPlayWnd()
{}

/// @brief Get pointer of playlist. 
Playlist* const DirPlayWnd::GetPlaylist()
{
    return &(impl_->playlist_);
}

/// @brief Constructor of DirPlayWndImpl.
/// @param mainWnd MainWnd
/// @param dirPlayWnd DirPlayWnd
/// @param titleWidth Width of title column.
DirPlayWnd::DirPlayWndImpl::DirPlayWndImpl(MainWnd& mainWnd, DirPlayWnd& dirPlayWnd, const unsigned int& titleWidth)
    : dirBrowseWnd_(0, false, mainWnd.GetLastBrowsedDirectory()),
      playlist_(mainWnd, "dir", titleWidth),
      mainWnd_(mainWnd)
{
    dirPlayWnd.add1(dirBrowseWnd_);

    scrolledWindow_.add(playlist_);
    scrolledWindow_.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    dirPlayWnd.add2(scrolledWindow_);

    dirBrowseWnd_.SignalSelected().connect(sigc::mem_fun(*this, &DirPlayWndImpl::OnSelected) );

    dirPlayWnd.set_position(229);

    dirPlayWnd.show_all_children();
}

/// @brief Callback function used when directory browser sent selected signal.
///
/// Remove all items in current playlist and add all items in selected directory
/// to playlist.
void DirPlayWnd::DirPlayWndImpl::OnSelected()
{
    playlist_.RemoveAll();

    mainWnd_.LogBrowsedDirectory(dirBrowseWnd_.SelectedDirectory());
    
    playlist_.AddDirectory(dirBrowseWnd_.SelectedDirectory(), false);
    
    playlist_.ForceUpdatePage();
}
