/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file main.cc
/// @brief main function file

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <ctime>

#include <vector>
#include <string>

#include <unistd.h>

#include <gtkmm/main.h>
#include <libintl.h>
#include "../include/mainmanager.h"
#include "../include/sockethandler.h"
#include "../include/trace.h"

/** @mainpage Main Page
    @section Intro
    Sagua is a music player for Gtk+/Gnome.

    If you want to look around pages, i recommend you to see @ref MainManager.
    @section Developer
    HG, Rhee <ez8tank@krpost.net>
    @section History
    - 2005-04-16 v0.2.3 released
    - 2005-03-03 v0.2.2 released
    - 2005-02-22 v0.2.1 released
    - 2005-02-12 v0.2.0 released
    - 2005-01-27 v0.1.2 released
    - 2005-01-20 v0.1.1 released
    - 2005-01-13 v0.1.0 released
    @section Requirement
    - Gtkmm 2.4 over
    - libao
    - libmad
    - libogg
    - libvorbis

    After v0.2.0
    - id3lib
	
    else
	    
    - libid3tag

    - Optional
    -# faad2
    -# ACE library
*/

/// @brief main function
///
/// Process command line arguments. If socket is connected to already executed
/// program, send arguments to that and exit. 
/// Or execute little server thread. After that, run MainManager loop.
int main(int argc, char* argv[])
{
    char opt = -1;
    extern int optind;
    
    while((opt = getopt(argc, argv, "dh")) != -1) {
	switch(opt) {
        case 'd':
            Tracer::activeTrace() = true;
            break;
        case 'h':
        default:
            std::cout << "Usage : " << argv[0] << " [-options] files " << std::endl << std::endl;
            std::cout << "Options : " << std::endl;
            std::cout << "  -d\tDisplay debug message(need build with debugtrace)." << std::endl;
            std::cout << "  -h\tDisplay this help message." << std::endl;	
            exit(0);
	}
    }
    Tracer::Trace trace("main");

    std::vector<std::string> arguments;
    if(optind < argc) {
	std::string eachArg;
	arguments.reserve(argc - optind);
	for(;optind < argc; ++optind) {

	    eachArg = argv[optind];
	    if(!Glib::path_is_absolute(eachArg)) {
		eachArg = Glib::get_current_dir();
		eachArg += '/';
		eachArg += argv[optind];
	    }
	    
	    arguments.push_back(eachArg);
	}
    }

    srand( time(NULL) );
    
    if(!Glib::thread_supported()) {
    	trace.Debug("thread init");

	Glib::thread_init();
    }

    UnixSocketHandler socketHandler(arguments);
    // If there exists socket already, send arguments and exit.
    // Or execute little server thread and go on. 
    if(socketHandler.Connect()) {

	socketHandler.SendArguments();
	
	exit(0);
	
    } else 
	socketHandler.StartServerLoop();

    Gtk::Main kit(argc, argv);

    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);

    MainManager* const mainManager = MainManager::Create(arguments, socketHandler);
    if(mainManager != NULL) { 
	socketHandler.SetMainManager(mainManager);

        mainManager->Run();
    }
    
    return 0;	
}
