/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file misc.cc
/// @brief Miscellaneous functions.

#include <fstream>

#include "../include/misc.h"

/// @brief Get extention part of filename.
std::string GetExt(const std::string& filename)
{
    const std::string::const_reverse_iterator coma(find_if(filename.rbegin(), filename.rend(), IsCharComa()));
    if(coma != filename.rend()) {

        std::string temp(coma.base(), find_if(coma.base(), filename.end(), IsCharSpace()));

        for_each(temp.begin(), temp.end(), ToLower());

        return temp;
    } else
        return std::string();
}

/// @brief Get filenames in directory and push to container.
/// @param path Directory to get filename.
/// @param dest Destination container.
/// @param recursive Get filenames recursively.
void GetFilenameFromDirToVector(const std::string& path, std::vector<std::string>& dest, bool recursive)
{
    Glib::Dir dir(path);

    std::string fullpath;
    // Very ugly repeat codes...
    if(recursive) {
        for(Glib::Dir::const_iterator eachFile = dir.begin(); eachFile != dir.end(); ++eachFile)
            {
                fullpath = path;
                fullpath += '/';
                fullpath += *eachFile;

                if(IsDirectory(fullpath)) {
                    // Repeat again or recursive call?
                    if(IsExecutable(fullpath) && IsReadable(fullpath)) {
                        Glib::Dir dir2(fullpath);
		    
                        std::string fullpath2;
                        for(Glib::Dir::const_iterator eachFile2 = dir2.begin(); eachFile2 != dir2.end(); ++eachFile2)
                            {
                                fullpath2 = fullpath;
                                fullpath2 += '/';
                                fullpath2 += *eachFile2;

                                if(IsDirectory(fullpath2)) {
                                    if(IsExecutable(fullpath2) && IsReadable(fullpath2))
                                        // If have more depth, call again anyway.
                                        GetFilenameFromDirToVector(fullpath2, dest, true);
                                } else
                                    dest.push_back(fullpath2);
                            }
                    }
                } else
                    dest.push_back(fullpath);
            }
    } else {
        for(Glib::Dir::const_iterator eachFile = dir.begin(); eachFile != dir.end(); ++eachFile)
            {
                fullpath = path;
                fullpath += '/';
                fullpath += *eachFile;

                if(!IsDirectory(fullpath))
                    dest.push_back(fullpath);
            }
    }

}

/// @brief Read config file and put to map.
/// @param filename Config filename.
/// @param configMap Destination map.
/// @param convertLowerCase Convert key and value to lower case.
/// @param valueBreakAtSpace Read value to space character. 
/// @return Success of open config file.
bool ConfigfileToMap(const std::string& filename, std::map<std::string, std::string>& configMap, const bool& convertLowerCase, const bool& valueBreakAtSpace)
{
    std::ifstream prefFile(filename.c_str());

    if(prefFile) {

        typedef std::string::iterator iter;

        std::string eachLine;

        while(std::getline(prefFile, eachLine)) {
            // Pass comment line, but there is no comment probably unless 
            // edited by user. Because i don't write comment line to config file. :)
            if(eachLine[0] == '#')
                continue;

            const iter end(eachLine.end());

            // Convert string to lower case
            if(convertLowerCase)
                for_each(eachLine.begin(), end, ToLower());

            // Eleminate front space character 
            iter i = find_if(eachLine.begin(), end, std::not1(IsCharSpace()));

            if(i != end) {

                // Find '=' character
                iter j = find_if(i, end, IsCharEqual());

                if(i != j) {
                    // Eleminate space character between i to j and that is key to insert.

                    std::string key(i, find_if(i, j, IsCharSpace()));

                    // Find value to insert.
                    i = find_if(++j, end, std::not1(IsCharSpace()));

                    if(i != end) {

                        if(valueBreakAtSpace)
                            j = find_if(i, end, IsCharSpace());
                        else
                            j = end;

                        configMap[key] = std::string(i, j);
                    }
                }
            }
        }

        return true;
    }

    return false;
}
