/* -*- Mode: C++; tab-width: 4; indent-tabs-mod: nil; c-basic-offset: 4 -*- */
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file prefwin.cc
/// @brief Class for user preference window source file.

#include <iostream>
#include <fstream>

#include <string>

#include <libintl.h>

#include <gtkmm/image.h>
#include <gtkmm/box.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>
#include <gtkmm/entry.h>
#include <gtkmm/radiobuttongroup.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/scale.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/tooltips.h>
#include <gtkmm/menu.h>
#include <gtkmm/optionmenu.h>
#include <gtkmm/combo.h>

#include "../include/fileentry.h"

#include "../include/mainmanager.h"
#include "../include/preference.h"
#include "../include/prefwin.h"
#include "../include/misc.h"
#include "../include/trace.h"

/// @brief General tab of Preference window.
class PrefWndGeneralTab : public Gtk::Table
{
public:
    PrefWndGeneralTab(const MainManager& mainManager, PrefWnd& prefWnd);
    virtual ~PrefWndGeneralTab() {}

    std::string GetCompactBgPixmapFilename() const
    {
        return Glib::filename_from_utf8(compactBgPixmap_.GetEntryText());
    }

    bool CompactBgPixmapChanged() const
    {
        return previousCompactBgPixmap_ != StringHash(Glib::filename_from_utf8(compactBgPixmap_.GetEntryText()));
    }

    const std::string& GetTrayIconAction() const
    {
        return selectedTrayIconAction_;
    }
	
    Glib::ustring GetMixerDev() const
    {
        return mixerDev_.get_text();
    }

    bool MixerDevChanged() const
    {
        return previousMixerDev_ != StringHash(mixerDev_.get_text());
    }

    Glib::ustring GetAssumeCharset() const
    {
        return assumeCharset_.get_text();
    }

#ifdef ENABLE_HTTP
    bool GetConvertUrl() const
    {
        return convertUrlBtn_.get_active();
    }
#endif

private:
    PrefWndGeneralTab(const PrefWndGeneralTab& prefWndGeneralTab);
    PrefWndGeneralTab& operator=(const PrefWndGeneralTab& prefWndGeneralTab);

    FileEntry compactBgPixmap_;

    Gtk::Menu trayIconActionMenu_;
    Gtk::OptionMenu trayIconActionOptionMenu_;

    Gtk::Entry mixerDev_;
    Gtk::Entry assumeCharset_;

    Gtk::Label titleLabel_;
    Gtk::Label trayIconActionLabel_;
    Gtk::Label assumeCharsetLabel_;
    Gtk::Label compactBgPixmapLabel_;
    Gtk::Label mixerDevLabel_;

#ifdef ENABLE_HTTP
    Gtk::CheckButton convertUrlBtn_;
#endif

    Gtk::Tooltips trayIconActionTooltips_;
    Gtk::Tooltips assumeCharsetTooltips_;

    std::string selectedTrayIconAction_;
    unsigned int previousCompactBgPixmap_;
    unsigned int previousMixerDev_;

    void OnMenuTrayIconAction(const std::string& action);

};

/// @brief Audio driver setting tab of Preference window.
class PrefWndAudioDriverTab : public Gtk::ScrolledWindow
{
public:
    PrefWndAudioDriverTab(MainManager& mainManager);
    virtual ~PrefWndAudioDriverTab() {}

    bool AudioDriverChanged() const;
	
    const std::string& GetSelectedAudioDriver() const
    {
        return selectedAudioDriver_;
    }
	
    Glib::ustring GetAlsa5Card() const
    {
        return alsa5Card_.get_text();
    }

    Glib::ustring GetAlsa5Dev() const
    {
        return alsa5Dev_.get_text();
    }

    Glib::ustring GetAlsa9Dev() const
    {
        return alsa9Dev_.get_text();
    }

    Glib::ustring GetOssDsp() const
    {
        return ossDsp_.get_text();
    }

    Glib::ustring GetSunDev() const
    {
        return sunDev_.get_text();
    }

    Glib::ustring GetAixsDev() const
    {
        return aixsDev_.get_text();
    }

private:
    PrefWndAudioDriverTab(const PrefWndAudioDriverTab& prefWndAudioDriverTab);
    PrefWndAudioDriverTab& operator=(const PrefWndAudioDriverTab& prefWndAudioDriverTab);

    MainManager& mainManager_;

    Gtk::Image warningIcon_;

    Gtk::HBox alsa5CardOptionBox_, alsa5DevOptionBox_;
    Gtk::HBox alsa9OptionBox_; 
    Gtk::HBox ossOptionBox_;
    Gtk::HBox sunOptionBox_, aixsOptionBox_;
    Gtk::HBox warningBox_;

    Gtk::Table mainTable_;

    Gtk::Entry alsa5Card_, alsa5Dev_;
    Gtk::Entry alsa9Dev_;
    Gtk::Entry ossDsp_;
    Gtk::Entry sunDev_;
    Gtk::Entry aixsDev_;

    Gtk::RadioButtonGroup driverBtnGroup_;
    Gtk::RadioButton defaultBtn_;
    Gtk::RadioButton alsa5Btn_;
    Gtk::RadioButton alsa9Btn_;
    Gtk::RadioButton esdBtn_;
    Gtk::RadioButton irixBtn_;
    Gtk::RadioButton ossBtn_;
    Gtk::RadioButton sunBtn_;
    Gtk::RadioButton aixsBtn_;

    Gtk::Button testBtn_;

    Gtk::Label titleLabel_;
    Gtk::Label alsa5CardLabel_;
    Gtk::Label alsa5DevLabel_;
    Gtk::Label alsa9DevLabel_;
    Gtk::Label ossDspLabel_;
    Gtk::Label sunDevLabel_;
    Gtk::Label aixsDevLabel_;
    Gtk::Label testLabel_;
    Gtk::Label testResultLabel_;
    Gtk::Label warningLabel_;

    Gtk::Tooltips alsa5Tooltips_;
    Gtk::Tooltips alsa9Tooltips_;
    Gtk::Tooltips alsa9DeviceTooltips_;

    std::string selectedAudioDriver_;
    unsigned int previousAudioDriver_;
    unsigned int previousAlsa5Card_;
    unsigned int previousAlsa5Dev_;
    unsigned int previousAlsa9Dev_;
    unsigned int previousOssDsp_;
    unsigned int previousSunDev_;
    unsigned int previousAixsDev_;

    void OnBtnAudioDriver(const char* driver);
    void OnBtnAudioDriverTest();
};

/// @brief Effector tab of Preference window.
class PrefWndEffectorTab : public Gtk::Table
{
public:
    PrefWndEffectorTab(const MainManager& mainManager);
    virtual ~PrefWndEffectorTab() {}

    bool GetEffector() const
    {
        return bassboostBtn_.get_active() || bassboost2Btn_.get_active() || \
            trebleboostBtn_.get_active() || stereoenhanceBtn_.get_active() || \
            rhythmeBtn_.get_active() || roomBtn_.get_active() || \
            roomBtn2_.get_active() || harmonyBtn_.get_active() || \
            distortionBtn_.get_active();
    }

    bool GetBassBoost() const
    {
        return bassboostBtn_.get_active();
    }

    bool GetBassBoost2() const
    {
        return bassboost2Btn_.get_active();
    }

    bool GetTrebleBoost() const
    {
        return trebleboostBtn_.get_active();
    }

    bool GetStereoEnhance() const
    {
        return stereoenhanceBtn_.get_active();
    }

    double GetStereoEnhanceAmount() const
    {
        return stereoenhanceScale_.get_value();
    }

    bool GetRhythme() const
    {
        return rhythmeBtn_.get_active();
    }

    double GetRhythmeAmount() const
    {
        return rhythmeScale_.get_value();
    }

    bool GetRoom() const
    {
        return roomBtn_.get_active();
    }

    double GetRoomAmount() const
    {
        return roomScale_.get_value();
    }

    bool GetRoom2() const
    {
        return roomBtn2_.get_active();
    }

    double GetRoom2Amount() const
    {
        return room2Scale_.get_value();
    }

    bool GetHarmony() const
    {
        return harmonyBtn_.get_active();
    }

    double GetHarmonyAmount() const
    {
        return harmonyScale_.get_value();
    }

    bool GetDistortion() const
    {
        return distortionBtn_.get_active();
    }

    double GetDistortionAmount() const
    {
        return distortionScale_.get_value();
    }

private:
    PrefWndEffectorTab(const PrefWndEffectorTab& prefWndEffectorTab);
    PrefWndEffectorTab& operator=(const PrefWndEffectorTab& prefWndEffectorTab);

    Gtk::HBox stereoenhanceBox_;
    Gtk::HBox roomBox_;
    Gtk::HBox room2Box_;
    Gtk::HBox harmonyBox_;
    Gtk::HBox rhythmeBox_;
    Gtk::HBox distortionBox_;

    Gtk::CheckButton bassboostBtn_;
    Gtk::CheckButton bassboost2Btn_;
    Gtk::CheckButton trebleboostBtn_;
    Gtk::CheckButton stereoenhanceBtn_; 
    Gtk::CheckButton rhythmeBtn_;
    Gtk::CheckButton roomBtn_;
    Gtk::CheckButton roomBtn2_;
    Gtk::CheckButton harmonyBtn_;
    Gtk::CheckButton distortionBtn_;

    Gtk::HScale stereoenhanceScale_;
    Gtk::HScale roomScale_;
    Gtk::HScale room2Scale_;
    Gtk::HScale harmonyScale_;
    Gtk::HScale rhythmeScale_;
    Gtk::HScale distortionScale_;

    Gtk::Label stableLabel_;
    Gtk::Label developLabel_; 

    Gtk::Tooltips bassBoostTooltips_;
    Gtk::Tooltips bassBoost2Tooltips_;
    Gtk::Tooltips trebleBoostTooltips_;
    Gtk::Tooltips stereoenhanceTooltips_;
    Gtk::Tooltips roomTooltips_;
    Gtk::Tooltips room2Tooltips_;

    void OnBtnStereoEnhance();
    void OnBtnRoom();
    void OnBtnRoom2();
    void OnBtnHarmony();
    void OnBtnRhythme();
    void OnBtnDistortion();
};

/// @brief GraphicEQ tab of Preference window.
class PrefWndGraphicEQTab : public Gtk::Table
{
public:
    PrefWndGraphicEQTab(const MainManager& mainManager);
    virtual ~PrefWndGraphicEQTab() {}

    bool GetGraphicEQ() const
    {
        return activateBtn_.get_active();
    }

    double GetEQ50() const
    {
        return EQ50_.get_value();
    }

    double GetEQ100() const
    {
        return EQ100_.get_value();
    }

    double GetEQ200() const
    {
        return EQ200_.get_value();
    }

    double GetEQ400() const
    {
        return EQ400_.get_value();
    }

    double GetEQ800() const
    {
        return EQ800_.get_value();
    }

    double GetEQ1500() const
    {
        return EQ1500_.get_value();
    }

    double GetEQ3k() const
    {
        return EQ3k_.get_value();
    }

    double GetEQ5k() const
    {
        return EQ5k_.get_value();
    }

    double GetEQ7k() const
    {
        return EQ7k_.get_value();
    }

    double GetEQ10k() const
    {
        return EQ10k_.get_value();
    }

    double GetEQ15k() const
    {
        return EQ15k_.get_value();
    }

private:
    PrefWndGraphicEQTab(const PrefWndGraphicEQTab& prefWndGraphicEQTab);
    PrefWndGraphicEQTab& operator=(const PrefWndGraphicEQTab& prefWndGraphicEQTab);

    Gtk::HBox EQBox50_, EQBox100_, EQBox200_, EQBox400_, EQBox800_, EQBox1500_;
    Gtk::HBox EQBox3k_, EQBox5k_, EQBox7k_, EQBox10k_, EQBox15k_;

    Gtk::CheckButton activateBtn_;

    Gtk::HScale EQ50_, EQ100_, EQ200_, EQ400_, EQ800_, EQ1500_;
    Gtk::HScale EQ3k_, EQ5k_, EQ7k_, EQ10k_, EQ15k_;

    Gtk::Label titleLabel_;

    void OnBtnGraphicEQ();

};

/// @brief Implementaion of PrefWnd.
class PrefWnd::PrefWndImpl
{
public:
    PrefWndImpl(MainManager& mainManager, PrefWnd& prefWnd);
    ~PrefWndImpl() {}

    MainManager& mainManager_;
    PrefWnd& prefWnd_;

    Gtk::VBox mainBox_;
    Gtk::Notebook mainNotebook_;
    Gtk::HButtonBox mainBtnBox_;

    Gtk::Button cancelBtn_, okBtn_;

    PrefWndGeneralTab prefWndGeneralTab_;
    PrefWndAudioDriverTab prefWndAudioDriverTab_;
    PrefWndEffectorTab prefWndEffectorTab_;
    PrefWndGraphicEQTab prefWndGraphicEQTab_;

    void OnBtnCancel();
    void OnBtnOk();
};

PrefWnd::PrefWnd(MainManager& mainManager)
    : impl_(new PrefWndImpl(mainManager, *this))
{}

PrefWnd::~PrefWnd()
{}

PrefWndGeneralTab::PrefWndGeneralTab(const MainManager& mainManager, PrefWnd& prefWnd)
    : compactBgPixmap_(prefWnd, gettext("Background image file used when main window is compact mode.")),
      titleLabel_(gettext("<span weight=\"bold\">General</span>"), 0.0, 0.5),
      trayIconActionLabel_(gettext("Tray icon click action : "), 0.0, 0.5),
      assumeCharsetLabel_(gettext("Assume charset : "), 0.0, 0.5),
      compactBgPixmapLabel_(gettext("Compact mode background : "), 0.0, 0.5),
      mixerDevLabel_(gettext("Mixer device : "), 0.0, 0.5),
#ifdef ENABLE_HTTP
      convertUrlBtn_(gettext("Convert URL to current encoding")),
#endif
      previousCompactBgPixmap_(0),
      previousMixerDev_(0)
{
    resize(3, 19);

    set_border_width(12);
    set_row_spacings(6);
    set_col_spacing(0, 12);
    set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    attach(titleLabel_, 0, 3, 0, 1, Gtk::FILL);

    attach(trayIconActionLabel_, 1, 2, 1, 2, Gtk::FILL);

    Gtk::Menu_Helpers::MenuList& list = trayIconActionMenu_.items();
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Open window"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "openwin") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("File info."),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "fileinfo") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Stop"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "stop") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Pause"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "pause") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Play"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "play") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Prev"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "prev") ) );
    list.push_back( Gtk::Menu_Helpers::MenuElem(gettext("Next"),
                                                sigc::bind( sigc::mem_fun(*this, &PrefWndGeneralTab::OnMenuTrayIconAction), "next") ) );

    selectedTrayIconAction_ = mainManager.GetPreference().GeneralTrayIconAction();
    if(selectedTrayIconAction_ == "openwin")
        trayIconActionMenu_.set_active(0);
    else if(selectedTrayIconAction_ == "fileinfo")
        trayIconActionMenu_.set_active(1);
    else if(selectedTrayIconAction_ == "stop")
        trayIconActionMenu_.set_active(2);
    else if(selectedTrayIconAction_ == "pause")
        trayIconActionMenu_.set_active(3);
    else if(selectedTrayIconAction_ == "play")
        trayIconActionMenu_.set_active(4);			
    else if(selectedTrayIconAction_ == "prev")
        trayIconActionMenu_.set_active(5);
    else if(selectedTrayIconAction_ == "next")
        trayIconActionMenu_.set_active(6);	
    else
        trayIconActionMenu_.set_active(0);

    trayIconActionOptionMenu_.set_menu(trayIconActionMenu_);
    trayIconActionTooltips_.set_tip(trayIconActionOptionMenu_, 
                                    gettext("Action when you click a tray icon in first button"));
    attach(trayIconActionOptionMenu_, 2, 3, 1, 2, Gtk::FILL);

    if(Glib::get_charset())
        assumeCharset_.set_text(mainManager.GetPreference().GeneralAssumeCharset());
    else
        assumeCharset_.set_sensitive(false);

    assumeCharsetTooltips_.set_tip(assumeCharset_,
                                   gettext("Assume charset when convert not valid UTF-8 string to UTF-8 and machine's charset is UTF-8"));	
    attach(assumeCharsetLabel_, 1, 2, 2, 3, Gtk::FILL);
    attach(assumeCharset_, 2, 3, 2, 3, Gtk::FILL); 

    attach(compactBgPixmapLabel_, 1, 2, 3, 4, Gtk::FILL);
    previousCompactBgPixmap_ = StringHash(mainManager.GetPreference().GeneralCompactBgPixmap());
    compactBgPixmap_.SetEntryText(mainManager.GetPreference().GeneralCompactBgPixmap());
    attach(compactBgPixmap_, 2, 3, 3, 4, Gtk::FILL);

    attach(mixerDevLabel_, 1, 2, 4, 5, Gtk::FILL);
    previousMixerDev_ = StringHash(mainManager.MixerDev());
    mixerDev_.set_text(mainManager.MixerDev());
    attach(mixerDev_, 2, 3, 4, 5, Gtk::FILL);

#ifdef ENABLE_HTTP
    convertUrlBtn_.set_active(mainManager.GetPreference().ConvertUrlToCurrentEncoding());
    if(Glib::get_charset())
        convertUrlBtn_.set_sensitive(false);

    attach(convertUrlBtn_, 1, 3, 5, 6, Gtk::FILL);
    attach(*manage(new Gtk::Label), 0, 3, 6, 19, Gtk::FILL);
#else
    attach(*manage(new Gtk::Label), 0, 3, 5, 19, Gtk::FILL);
#endif
}

/// @brief Callback function used when trayicon action option menu is clicked.
void PrefWndGeneralTab::OnMenuTrayIconAction(const std::string& action)
{
    selectedTrayIconAction_ = action;
}

PrefWndAudioDriverTab::PrefWndAudioDriverTab(MainManager& mainManager)
    : mainManager_(mainManager),
      warningIcon_(Gtk::Stock::DIALOG_WARNING, Gtk::ICON_SIZE_LARGE_TOOLBAR),
      mainTable_(5, 19),
      defaultBtn_(driverBtnGroup_, gettext("Use default driver\n(auto detect)")),
      alsa5Btn_(driverBtnGroup_, "Alsa 0.5.x"),
      alsa9Btn_(driverBtnGroup_, "Alsa 0.9.x"),
      esdBtn_(driverBtnGroup_, "ESD"),
      irixBtn_(driverBtnGroup_, "Irix"),
      ossBtn_(driverBtnGroup_, "OSS"),
      sunBtn_(driverBtnGroup_, "SUN"),
      aixsBtn_(driverBtnGroup_, "Aixs"),
      testBtn_(gettext("Test")),
      titleLabel_(gettext("<span weight=\"bold\">Audio Driver</span>"), 0.0, 0.5),
      alsa5CardLabel_("Card : ", 0.0, 0.5),
      alsa5DevLabel_("Device : ", 0.0, 0.5),
      alsa9DevLabel_("Device : ", 0.0, 0.5),
      ossDspLabel_(gettext("Path : "), 0.0, 0.5),
      sunDevLabel_(gettext("Path : "), 0.0, 0.5),
      aixsDevLabel_(gettext("Path : "), 0.0, 0.5),
      testLabel_(gettext("<span weight=\"bold\">Audio Driver Test</span>"), 0.0, 0.5),
      warningLabel_(gettext("Change will stop your playing!")),
      previousAudioDriver_(0),
      previousAlsa5Card_(0),
      previousAlsa5Dev_(0),
      previousAlsa9Dev_(0),
      previousOssDsp_(0),
      previousSunDev_(0),
      previousAixsDev_(0)
{
    mainTable_.set_border_width(12);
    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    mainTable_.attach(titleLabel_, 0, 5, 0, 1, Gtk::FILL);

    warningBox_.pack_start(warningIcon_, Gtk::PACK_SHRINK);
    warningBox_.pack_start(warningLabel_, Gtk::PACK_SHRINK);
    mainTable_.attach(warningBox_, 1, 5, 1, 2, Gtk::FILL);

    previousAudioDriver_ = StringHash(mainManager.GetPreference().AudioDriver());
    selectedAudioDriver_ = mainManager.GetPreference().AudioDriver();

    defaultBtn_.signal_clicked().connect( sigc::bind(
                                                     sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "default") );
    mainTable_.attach(defaultBtn_, 1, 4, 2, 3, Gtk::FILL);
    mainTable_.attach(*manage(new Gtk::Label), 4, 5, 2, 3, Gtk::EXPAND);

    alsa5Btn_.signal_clicked().connect( sigc::bind(
                                                   sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "alsa") );
    alsa5Tooltips_.set_tip(alsa5Btn_, gettext("Driver for alsa 0.5.x"));
    mainTable_.attach(alsa5Btn_, 1, 4, 3, 4, Gtk::FILL);

    mainTable_.attach(alsa5CardLabel_, 2, 3, 4, 5, Gtk::FILL);
    alsa5CardOptionBox_.set_spacing(12);
    alsa5CardOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa5Card_ = StringHash(mainManager.GetPreference().AudioAlsa5Card());
    alsa5Card_.set_text(mainManager.GetPreference().AudioAlsa5Card());
    alsa5CardOptionBox_.pack_start(alsa5Card_);
    mainTable_.attach(alsa5CardOptionBox_, 3, 5, 4, 5, Gtk::FILL);

    mainTable_.attach(alsa5DevLabel_, 2, 3, 5, 6, Gtk::FILL);
    alsa5DevOptionBox_.set_spacing(12);
    alsa5DevOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa5Dev_ = StringHash(mainManager.GetPreference().AudioAlsa5Dev());
    alsa5Dev_.set_text(mainManager.GetPreference().AudioAlsa5Dev());
    alsa5DevOptionBox_.pack_start(alsa5Dev_);
    mainTable_.attach(alsa5DevOptionBox_, 3, 5, 5, 6, Gtk::FILL);

    alsa9Btn_.signal_clicked().connect( sigc::bind(
                                                   sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "alsa09") );
    alsa9Tooltips_.set_tip(alsa9Btn_, gettext("Driver for alsa 0.9.x and 1.0.x"));
    mainTable_.attach(alsa9Btn_, 1, 4, 6, 7, Gtk::FILL);

    mainTable_.attach(alsa9DevLabel_, 2, 3, 7, 8, Gtk::FILL);
    alsa9OptionBox_.set_spacing(12);
    alsa9OptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAlsa9Dev_ = StringHash(mainManager.GetPreference().AudioAlsa9Dev());
    alsa9Dev_.set_text(mainManager.GetPreference().AudioAlsa9Dev());
    alsa9DeviceTooltips_.set_tip(alsa9Dev_, gettext("ex) hw:0,0 or ens1371(naming in asoundrc)"));
    alsa9OptionBox_.pack_start(alsa9Dev_);
    mainTable_.attach(alsa9OptionBox_, 3, 5, 7, 8, Gtk::FILL);

    esdBtn_.signal_clicked().connect( sigc::bind(
                                                 sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "esd") );	
    mainTable_.attach(esdBtn_, 1, 4, 8, 9, Gtk::FILL);

    irixBtn_.signal_clicked().connect( sigc::bind(
                                                  sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "irix") );
    mainTable_.attach(irixBtn_, 1, 4, 9, 10, Gtk::FILL);

    ossBtn_.signal_clicked().connect( sigc::bind(
                                                 sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "oss") );
    mainTable_.attach(ossBtn_, 1, 4, 10, 11, Gtk::FILL);

    mainTable_.attach(ossDspLabel_, 2, 3, 11, 12, Gtk::FILL);
    ossOptionBox_.set_spacing(12);
    ossOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousOssDsp_ = StringHash(mainManager.GetPreference().AudioOssDsp());
    ossDsp_.set_text(mainManager.GetPreference().AudioOssDsp());
    ossOptionBox_.pack_start(ossDsp_);
    mainTable_.attach(ossOptionBox_, 3, 5, 11, 12, Gtk::FILL);

    sunBtn_.signal_clicked().connect( sigc::bind(
                                                 sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "sun") );
    mainTable_.attach(sunBtn_, 1, 4, 12, 13, Gtk::FILL);

    mainTable_.attach(sunDevLabel_, 2, 3, 13, 14, Gtk::FILL);
    sunOptionBox_.set_spacing(12);
    sunOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousSunDev_ = StringHash(mainManager.GetPreference().AudioSunDev());
    sunDev_.set_text(mainManager.GetPreference().AudioSunDev());
    sunOptionBox_.pack_start(sunDev_);
    mainTable_.attach(sunOptionBox_, 3, 5, 13, 14, Gtk::FILL);

    aixsBtn_.signal_clicked().connect( sigc::bind(
                                                  sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriver), "aixs") );
    mainTable_.attach(aixsBtn_, 1, 4, 14, 15, Gtk::FILL);

    mainTable_.attach(aixsDevLabel_, 2, 3, 15, 16, Gtk::FILL);
    aixsOptionBox_.set_spacing(12);
    aixsOptionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    previousAixsDev_ = StringHash(mainManager.GetPreference().AudioAixsDev());
    aixsDev_.set_text(mainManager.GetPreference().AudioAixsDev());
    aixsOptionBox_.pack_start(aixsDev_);
    mainTable_.attach(aixsOptionBox_, 3, 5, 15, 16, Gtk::FILL);

    testLabel_.set_use_markup(true);
    mainTable_.attach(testLabel_, 0, 5, 17, 18, Gtk::FILL);

    testBtn_.signal_clicked().connect(
                                      sigc::mem_fun(*this, &PrefWndAudioDriverTab::OnBtnAudioDriverTest) );
    mainTable_.attach(testBtn_, 1, 3, 18, 19, Gtk::FILL);
    mainTable_.attach(testResultLabel_, 3, 5, 18, 19, Gtk::FILL);

    alsa5CardOptionBox_.set_sensitive(false);
    alsa5DevOptionBox_.set_sensitive(false);
    alsa9OptionBox_.set_sensitive(false);
    ossOptionBox_.set_sensitive(false);
    sunOptionBox_.set_sensitive(false);
    aixsOptionBox_.set_sensitive(false);

    if(selectedAudioDriver_ == "alsa") {
        alsa5Btn_.set_active(true);
        alsa5CardOptionBox_.set_sensitive(true);
        alsa5DevOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "alsa09") {
        alsa9Btn_.set_active(true);
        alsa9OptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "esd") {
        esdBtn_.set_active(true);
    } else if(selectedAudioDriver_ == "irix") {
        irixBtn_.set_active(true);
    } else if(selectedAudioDriver_ == "oss") {
        ossBtn_.set_active(true);
        ossOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "sun") {
        sunBtn_.set_active(true);
        sunOptionBox_.set_sensitive(true);
    } else if(selectedAudioDriver_ == "aixs") {
        aixsBtn_.set_active(true);
        aixsOptionBox_.set_sensitive(true);
    } else
        defaultBtn_.set_active(true);

    set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
    add(mainTable_);
}

bool PrefWndAudioDriverTab::AudioDriverChanged() const
{
    if(StringHash(selectedAudioDriver_) != previousAudioDriver_) {

        return true;

    } else {
        if(selectedAudioDriver_ == "alsa") {

            if(StringHash(alsa5Card_.get_text()) != previousAlsa5Card_ ||
               StringHash(alsa5Dev_.get_text()) != previousAlsa5Dev_)
                return true;

        } else if(selectedAudioDriver_ == "alsa09") {

            if(StringHash(alsa9Dev_.get_text()) != previousAlsa9Dev_)
                return true;

        } else if(selectedAudioDriver_ == "oss") {

            if(StringHash(ossDsp_.get_text()) != previousOssDsp_)
                return true;

        } else if(selectedAudioDriver_ == "sun") {

            if(StringHash(sunDev_.get_text()) != previousSunDev_)
                return true;

        } else if(selectedAudioDriver_ == "aixs") {

            if(StringHash(aixsDev_.get_text()) != previousAixsDev_)
                return true;
        }

    }

    return false;
}

/// @brief Callback function used when any audio driver check button is clicked.
void PrefWndAudioDriverTab::OnBtnAudioDriver(const char* driver)
{
    selectedAudioDriver_ = driver;

    alsa5CardOptionBox_.set_sensitive(false);
    alsa5DevOptionBox_.set_sensitive(false);
    alsa9OptionBox_.set_sensitive(false);
    ossOptionBox_.set_sensitive(false);
    sunOptionBox_.set_sensitive(false);
    aixsOptionBox_.set_sensitive(false);

    if(selectedAudioDriver_ == "alsa") {

        alsa5CardOptionBox_.set_sensitive(true);
        alsa5DevOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "alsa09") {

        alsa9OptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "oss") {

        ossOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "sun") {

        sunOptionBox_.set_sensitive(true);

    } else if(selectedAudioDriver_ == "aixs") {

        aixsOptionBox_.set_sensitive(true);

    }
}

/// @brief Callback function used when audio driver test button is clicked.
/// @sa MainManager::AudioTestDriver
void PrefWndAudioDriverTab::OnBtnAudioDriverTest()
{
    std::map<std::string, std::string> option;

    if(selectedAudioDriver_ == "alsa") {
        option["card"] = alsa5Card_.get_text();
        option["dev"] = alsa5Dev_.get_text();

    } else if(selectedAudioDriver_ == "alsa09") {

        option["dev"] = alsa9Dev_.get_text();	

    } else if(selectedAudioDriver_ == "oss") {

        option["dsp"] = ossDsp_.get_text();

    } else if(selectedAudioDriver_ == "sun") {

        option["dev"] = sunDev_.get_text();

    } else if(selectedAudioDriver_ == "aixs") {

        option["dev"] = aixsDev_.get_text();

    }

    if(mainManager_.AudioTestDriver(selectedAudioDriver_.c_str(), option)) 
        testResultLabel_.set_text(gettext("Success"));
    else
        testResultLabel_.set_text(gettext("Fail"));	

}

PrefWndEffectorTab::PrefWndEffectorTab(const MainManager& mainManager)
    : bassboostBtn_("Bass Boost"),
      bassboost2Btn_("Bass Boost2"),
      trebleboostBtn_("High Frequency Boost"),
      stereoenhanceBtn_("Stereo Enhance"),
      rhythmeBtn_("Rhythme Effect"),
      roomBtn_("Room Effect"),
      roomBtn2_("Room2 Effect"),
      harmonyBtn_("Harmony Effect"),
      distortionBtn_("Distortion Effect"),
      stereoenhanceScale_(0.1, 2.1, 0.1),
      roomScale_(0.1, 1.1, 0.1),
      room2Scale_(0.1, 1.1, 0.1),
      harmonyScale_(0.1, 1.1, 0.1),
      rhythmeScale_(0.1, 1.6, 0.1),
      distortionScale_(1.0, 10.1, 0.1),
      stableLabel_(gettext("<span weight=\"bold\">Effector</span>"), 0.0, 0.5),
      developLabel_(gettext("<span weight=\"bold\">Experimental Effector</span>"), 0.0, 0.5)
{
    resize(3, 19);

    set_border_width(12);
    set_row_spacings(6);
    set_col_spacing(0, 12);
    set_col_spacing(1, 12);

    stableLabel_.set_use_markup(true);
    attach(stableLabel_, 0, 3, 0, 1, Gtk::FILL);

    bassboostBtn_.set_active(mainManager.GetPreference().EfxBassBoost());
    bassBoostTooltips_.set_tip(bassboostBtn_, gettext("Boost low frequency band"));
    attach(bassboostBtn_, 1, 2, 1, 2, Gtk::FILL);
    attach(*manage(new Gtk::Label), 2, 3, 1, 2, Gtk::EXPAND);

    stereoenhanceBtn_.set_active(mainManager.GetPreference().EfxStereoEnhance());
    stereoenhanceBtn_.signal_clicked().connect(
                                               sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnStereoEnhance) );
    stereoenhanceTooltips_.set_tip(stereoenhanceBtn_, gettext("Stereo enhance effect"));
    attach(stereoenhanceBtn_, 1, 2, 2, 3, Gtk::FILL);
    stereoenhanceScale_.set_value_pos(Gtk::POS_RIGHT);
    stereoenhanceScale_.set_draw_value();
    stereoenhanceScale_.set_value(mainManager.GetPreference().EfxStereoEnhanceAmount());
    stereoenhanceScale_.set_sensitive(mainManager.GetPreference().EfxStereoEnhance());
    stereoenhanceBox_.set_spacing(12);
    stereoenhanceBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    stereoenhanceBox_.pack_start(stereoenhanceScale_);
    attach(stereoenhanceBox_, 1, 3, 3, 4, Gtk::FILL);
    set_row_spacing(2, 3);

    attach(*manage(new Gtk::Label), 0, 3, 4, 5, Gtk::FILL);

    developLabel_.set_use_markup(true);
    attach(developLabel_, 0, 3, 5, 6, Gtk::FILL);

    bassboost2Btn_.set_active(mainManager.GetPreference().EfxBassBoost2());
    bassBoost2Tooltips_.set_tip(bassboost2Btn_, gettext("Experimental low frequency band boost"));
    attach(bassboost2Btn_, 1, 2, 6, 7, Gtk::FILL);

    trebleboostBtn_.set_active(mainManager.GetPreference().EfxTrebleBoost());
    trebleBoostTooltips_.set_tip(trebleboostBtn_, 
                                 gettext("Boost high frequency band, also you can here high frequency noise"));
    attach(trebleboostBtn_, 1, 2, 7, 8, Gtk::FILL);

    roomBtn_.set_active(mainManager.GetPreference().EfxRoom());
    roomBtn_.signal_clicked().connect(
                                      sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRoom) );
    roomTooltips_.set_tip(roomBtn_,
                          gettext("Experimental space effector"));
    attach(roomBtn_, 1, 2, 8, 9, Gtk::FILL);
    roomScale_.set_value_pos(Gtk::POS_RIGHT);
    roomScale_.set_draw_value();
    roomScale_.set_value(mainManager.GetPreference().EfxRoomAmount());
    roomScale_.set_sensitive(mainManager.GetPreference().EfxRoom());
    roomBox_.set_spacing(12);
    roomBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    roomBox_.pack_start(roomScale_);
    attach(roomBox_, 1, 3, 9, 10, Gtk::FILL);
    set_row_spacing(8, 3);

    roomBtn2_.set_active(mainManager.GetPreference().EfxRoom2());
    roomBtn2_.signal_clicked().connect(
                                       sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRoom2) );
    room2Tooltips_.set_tip(roomBtn2_,
                           gettext("Another experimental space effector"));
    attach(roomBtn2_, 1, 2, 10, 11, Gtk::FILL);
    room2Scale_.set_value_pos(Gtk::POS_RIGHT);
    room2Scale_.set_draw_value();
    room2Scale_.set_value(mainManager.GetPreference().EfxRoom2Amount());
    room2Scale_.set_sensitive(mainManager.GetPreference().EfxRoom2());
    room2Box_.set_spacing(12);
    room2Box_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    room2Box_.pack_start(room2Scale_);
    attach(room2Box_, 1, 3, 11, 12, Gtk::FILL);
    set_row_spacing(10, 3);

    harmonyBtn_.set_active(mainManager.GetPreference().EfxHarmony());
    harmonyBtn_.signal_clicked().connect(
                                         sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnHarmony) );
    attach(harmonyBtn_, 1, 2, 12, 13, Gtk::FILL);
    harmonyScale_.set_value_pos(Gtk::POS_RIGHT);
    harmonyScale_.set_draw_value();
    harmonyScale_.set_value(mainManager.GetPreference().EfxHarmonyAmount());
    harmonyScale_.set_sensitive(mainManager.GetPreference().EfxHarmony());
    harmonyBox_.set_spacing(12);
    harmonyBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    harmonyBox_.pack_start(harmonyScale_);
    attach(harmonyBox_, 1, 3, 13, 14, Gtk::FILL);
    set_row_spacing(12, 3);

    rhythmeBtn_.set_active(mainManager.GetPreference().EfxRhythme());
    rhythmeBtn_.signal_clicked().connect(
                                         sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnRhythme) );
    attach(rhythmeBtn_, 1, 2, 14, 15, Gtk::FILL);
    rhythmeScale_.set_value_pos(Gtk::POS_RIGHT);
    rhythmeScale_.set_draw_value();
    rhythmeScale_.set_value(mainManager.GetPreference().EfxRhythmeAmount());
    rhythmeScale_.set_sensitive(mainManager.GetPreference().EfxRhythme());
    rhythmeBox_.set_spacing(12);
    rhythmeBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    rhythmeBox_.pack_start(rhythmeScale_);
    attach(rhythmeBox_, 1, 3, 15, 16, Gtk::FILL);
    set_row_spacing(14, 3);

    distortionBtn_.set_active(mainManager.GetPreference().EfxDistortion());
    distortionBtn_.signal_clicked().connect(
                                            sigc::mem_fun(*this, &PrefWndEffectorTab::OnBtnDistortion) );
    attach(distortionBtn_, 1, 2, 16, 17, Gtk::FILL);
    distortionScale_.set_value_pos(Gtk::POS_RIGHT);
    distortionScale_.set_draw_value();
    distortionScale_.set_value(mainManager.GetPreference().EfxDistortionAmount());
    distortionScale_.set_sensitive(mainManager.GetPreference().EfxDistortion());
    distortionBox_.set_spacing(12);
    distortionBox_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    distortionBox_.pack_start(distortionScale_);
    attach(distortionBox_, 1, 3, 17, 18, Gtk::FILL);
    set_row_spacing(16, 3);

    attach(*manage(new Gtk::Label), 0, 3, 18, 19, Gtk::FILL);
}

/// @brief Callback function used when stereo enhance effector check button is clicked.
void PrefWndEffectorTab::OnBtnStereoEnhance()
{
    if(!stereoenhanceBtn_.get_active())
        stereoenhanceScale_.set_sensitive(false);
    else
        stereoenhanceScale_.set_sensitive(true);	
}

/// @brief Callback function used when room effector check button is clicked.
void PrefWndEffectorTab::OnBtnRoom()
{
    if(!roomBtn_.get_active())
        roomScale_.set_sensitive(false);
    else
        roomScale_.set_sensitive(true);
}

/// @brief Callback function used when room2 effector check button is clicked.
void PrefWndEffectorTab::OnBtnRoom2()
{
    if(!roomBtn2_.get_active())
        room2Scale_.set_sensitive(false);
    else
        room2Scale_.set_sensitive(true);
}

/// @brief Callback function used when harmony effector check button is clicked.
void PrefWndEffectorTab::OnBtnHarmony()
{
    if(!harmonyBtn_.get_active())
        harmonyScale_.set_sensitive(false);
    else
        harmonyScale_.set_sensitive(true);	
}

/// @brief Callback function used when rhythme effector check button is clicked.
void PrefWndEffectorTab::OnBtnRhythme()
{
    if(!rhythmeBtn_.get_active())
        rhythmeScale_.set_sensitive(false);
    else
        rhythmeScale_.set_sensitive(true);	
}

/// @brief Callback function used when distortion effector check button is clicked.
void PrefWndEffectorTab::OnBtnDistortion()
{
    if(!distortionBtn_.get_active())
        distortionScale_.set_sensitive(false);
    else
        distortionScale_.set_sensitive(true);	
}

PrefWndGraphicEQTab::PrefWndGraphicEQTab(const MainManager& mainManager)
    : activateBtn_( gettext("Use equalizer") ),
      // Is it efficient more than 6 dB? anyway...
      EQ50_(-12.0, 12.1, 0.1),
      EQ100_(-12.0, 12.1, 0.1),
      EQ200_(-12.0, 12.1, 0.1),
      EQ400_(-12.0, 12.1, 0.1),
      EQ800_(-12.0, 12.1, 0.1),
      EQ1500_(-12.0, 12.1, 0.1),
      EQ3k_(-12.0, 12.1, 0.1),
      EQ5k_(-12.0, 12.1, 0.1),
      EQ7k_(-12.0, 12.1, 0.1),
      EQ10k_(-12.0, 12.1, 0.1),
      EQ15k_(-12.0, 12.1, 0.1),
      titleLabel_(gettext("<span weight=\"bold\">Equalizer</span>"), 0.0, 0.5)
{
    resize(5, 19);

    set_border_width(12);
    set_row_spacings(6);
    set_col_spacing(0, 12);
    set_col_spacing(1, 12);

    titleLabel_.set_use_markup(true);
    attach(titleLabel_, 0, 5, 0, 1, Gtk::FILL);

    activateBtn_.set_active(mainManager.GetPreference().EfxGraphicEQ());
    activateBtn_.signal_clicked().connect(
                                          sigc::mem_fun(*this, &PrefWndGraphicEQTab::OnBtnGraphicEQ) );
    attach(activateBtn_, 1, 4, 1, 2, Gtk::FILL);
    attach(*manage(new Gtk::Label), 4, 5, 1, 2, Gtk::EXPAND);

    EQ50_.set_value_pos(Gtk::POS_RIGHT);
    EQ50_.set_draw_value();
    EQ50_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[0] );
    EQ50_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox50_.set_spacing(12);
    EQBox50_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox50_.pack_start(EQ50_);
    attach(*manage(new Gtk::Label("50 Hz", 0.0, 0.5)), 2, 3, 2, 3, Gtk::FILL);
    attach(EQBox50_, 3, 5, 2, 3, Gtk::FILL);

    EQ100_.set_value_pos(Gtk::POS_RIGHT);
    EQ100_.set_draw_value();
    EQ100_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[1] );
    EQ100_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox100_.set_spacing(12);
    EQBox100_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox100_.pack_start(EQ100_);
    attach(*manage(new Gtk::Label("100 Hz", 0.0, 0.5)), 2, 3, 3, 4, Gtk::FILL);
    attach(EQBox100_, 3, 5, 3, 4, Gtk::FILL);

    EQ200_.set_value_pos(Gtk::POS_RIGHT);
    EQ200_.set_draw_value();
    EQ200_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[2] );
    EQ200_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox200_.set_spacing(12);
    EQBox200_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox200_.pack_start(EQ200_);
    attach(*manage(new Gtk::Label("200 Hz", 0.0, 0.5)), 2, 3, 4, 5, Gtk::FILL);
    attach(EQBox200_, 3, 5, 4, 5, Gtk::FILL);

    EQ400_.set_value_pos(Gtk::POS_RIGHT);
    EQ400_.set_draw_value();
    EQ400_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[3] );
    EQ400_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox400_.set_spacing(12);
    EQBox400_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox400_.pack_start(EQ400_);
    attach(*manage(new Gtk::Label("400 Hz", 0.0, 0.5)), 2, 3, 5, 6, Gtk::FILL);
    attach(EQBox400_, 3, 5, 5, 6, Gtk::FILL);

    EQ800_.set_value_pos(Gtk::POS_RIGHT);
    EQ800_.set_draw_value();
    EQ800_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[4] );
    EQ800_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox800_.set_spacing(12);
    EQBox800_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox800_.pack_start(EQ800_);
    attach(*manage(new Gtk::Label("800 Hz", 0.0, 0.5)), 2, 3, 6, 7, Gtk::FILL);
    attach(EQBox800_, 3, 5, 6, 7, Gtk::FILL);

    EQ1500_.set_value_pos(Gtk::POS_RIGHT);
    EQ1500_.set_draw_value();
    EQ1500_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[5] );
    EQ1500_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox1500_.set_spacing(12);
    EQBox1500_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox1500_.pack_start(EQ1500_);
    attach(*manage(new Gtk::Label("1.5 kHz", 0.0, 0.5)), 2, 3, 7, 8, Gtk::FILL);
    attach(EQBox1500_, 3, 5, 7, 8, Gtk::FILL);

    EQ3k_.set_value_pos(Gtk::POS_RIGHT);
    EQ3k_.set_draw_value();
    EQ3k_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[6] );
    EQ3k_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox3k_.set_spacing(12);
    EQBox3k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox3k_.pack_start(EQ3k_);
    attach(*manage(new Gtk::Label("3 kHz", 0.0, 0.5)), 2, 3, 8, 9, Gtk::FILL);
    attach(EQBox3k_, 3, 5, 8, 9, Gtk::FILL);

    EQ5k_.set_value_pos(Gtk::POS_RIGHT);
    EQ5k_.set_draw_value();
    EQ5k_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[7] );
    EQ5k_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox5k_.set_spacing(12);
    EQBox5k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox5k_.pack_start(EQ5k_);
    attach(*manage(new Gtk::Label("5 kHz", 0.0, 0.5)), 2, 3, 9, 10, Gtk::FILL);
    attach(EQBox5k_, 3, 5, 9, 10, Gtk::FILL);

    EQ7k_.set_value_pos(Gtk::POS_RIGHT);
    EQ7k_.set_draw_value();
    EQ7k_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[8] );
    EQ7k_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox7k_.set_spacing(12);
    EQBox7k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox7k_.pack_start(EQ7k_);
    attach(*manage(new Gtk::Label("7 kHz", 0.0, 0.5)), 2, 3, 10, 11, Gtk::FILL);
    attach(EQBox7k_, 3, 5, 10, 11, Gtk::FILL);

    EQ10k_.set_value_pos(Gtk::POS_RIGHT);
    EQ10k_.set_draw_value();
    EQ10k_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[9] );
    EQ10k_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox10k_.set_spacing(12);
    EQBox10k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox10k_.pack_start(EQ10k_);
    attach(*manage(new Gtk::Label("10 kHz", 0.0, 0.5)), 2, 3, 11, 12, Gtk::FILL);
    attach(EQBox10k_, 3, 5, 11, 12, Gtk::FILL);

    EQ15k_.set_value_pos(Gtk::POS_RIGHT);
    EQ15k_.set_draw_value();
    EQ15k_.set_value( (mainManager.GetPreference().EfxGraphicEQGain())[10] );
    EQ15k_.set_sensitive(mainManager.GetPreference().EfxGraphicEQ());
    EQBox15k_.set_spacing(12);
    EQBox15k_.pack_start(*manage(new Gtk::Label), Gtk::PACK_SHRINK);
    EQBox15k_.pack_start(EQ15k_);
    attach(*manage(new Gtk::Label("15 kHz", 0.0, 0.5)), 2, 3, 12, 13, Gtk::FILL);
    attach(EQBox15k_, 3, 5, 12, 13, Gtk::FILL);

    attach(*manage(new Gtk::Label), 0, 5, 13, 19, Gtk::FILL);
}

/// @brief Callback function used when graphic EQ check button is clicked.
void PrefWndGraphicEQTab::OnBtnGraphicEQ()
{
    if(activateBtn_.get_active()) {
        EQ50_.set_sensitive(true);
        EQ100_.set_sensitive(true);
        EQ200_.set_sensitive(true);
        EQ400_.set_sensitive(true);
        EQ800_.set_sensitive(true);
        EQ1500_.set_sensitive(true);
        EQ3k_.set_sensitive(true);
        EQ5k_.set_sensitive(true);
        EQ7k_.set_sensitive(true);
        EQ10k_.set_sensitive(true);
        EQ15k_.set_sensitive(true);
    } else {
        EQ50_.set_sensitive(false);
        EQ100_.set_sensitive(false);
        EQ200_.set_sensitive(false);
        EQ400_.set_sensitive(false);
        EQ800_.set_sensitive(false);
        EQ1500_.set_sensitive(false);
        EQ3k_.set_sensitive(false);
        EQ7k_.set_sensitive(false);
        EQ5k_.set_sensitive(false);
        EQ10k_.set_sensitive(false);
        EQ15k_.set_sensitive(false);
    }
}

PrefWnd::PrefWndImpl::PrefWndImpl(MainManager& mainManager, PrefWnd& prefWnd)
    : mainManager_(mainManager),
      prefWnd_(prefWnd),
      mainBtnBox_(Gtk::BUTTONBOX_END, 12),
      cancelBtn_(Gtk::Stock::CANCEL),
      okBtn_(Gtk::Stock::OK),
      prefWndGeneralTab_(mainManager, prefWnd),
      prefWndAudioDriverTab_(mainManager),
      prefWndEffectorTab_(mainManager),
      prefWndGraphicEQTab_(mainManager)
{
    prefWnd_.set_title(gettext("Preference"));
    prefWnd_.set_resizable(false);

    mainBox_.set_border_width(12);
    mainBox_.set_spacing(12);
    prefWnd_.add(mainBox_);

    mainNotebook_.append_page(prefWndGeneralTab_, gettext("General"));
    mainNotebook_.append_page(prefWndAudioDriverTab_, gettext("Audio Driver"));
    mainNotebook_.append_page(prefWndEffectorTab_, gettext("Effector"));
    mainNotebook_.append_page(prefWndGraphicEQTab_, gettext("Equalizer"));

    mainBox_.pack_start(mainNotebook_);

    cancelBtn_.signal_clicked().connect(
                                        sigc::mem_fun(*this, &PrefWndImpl::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);
    okBtn_.signal_clicked().connect(
                                    sigc::mem_fun(*this, &PrefWndImpl::OnBtnOk) );
    mainBtnBox_.pack_start(okBtn_);

    mainBox_.pack_start(mainBtnBox_);

    prefWnd_.show_all_children();
}

/// @brief Callback function used when cancel button is clicked.
void PrefWnd::PrefWndImpl::OnBtnCancel()
{
    prefWnd_.hide();
}

/// @brief Callback function used when ok button is clicked.
///
/// Apply to current Preference and change audio driver if need.
void PrefWnd::PrefWndImpl::OnBtnOk()
{
    Tracer::Trace trace("PrefWnd::PrefWndImpl::OnBtnOk");

    if(mainManager_.CheckProgramDir()) {
	
        std::ofstream prefFile(mainManager_.GetProgramDirFilePath("/config").c_str());

        if(prefFile) 
            prefFile << *this; 
        else
            trace.Debug("Can't write prefFile");	

    }

    mainManager_.GetWritablePreference() << prefWnd_;

    if(prefWndGeneralTab_.MixerDevChanged())
        mainManager_.MixerRefresh();

    if(mainManager_.GetPreference().Effect())
        mainManager_.GetPreference().EfxChangedSet();

    if(prefWndAudioDriverTab_.AudioDriverChanged()) {
        trace.Debug("Audio driver changed, i'll try set new audio driver");
        mainManager_.AudioSetChangedDriver();
    }

    if(prefWndGeneralTab_.CompactBgPixmapChanged())
        mainManager_.MainWndChangeCompactBgPixmap();

    prefWnd_.hide();
}

/// @brief Apply to Preference.
void operator<<(Preference& preference, const PrefWnd& prefWnd)
{
    preference.generalTrayIconAction_ = prefWnd.impl_->prefWndGeneralTab_.GetTrayIconAction();
    preference.generalAssumeCharset_ = prefWnd.impl_->prefWndGeneralTab_.GetAssumeCharset();
    preference.generalCompactBgPixmap_ = prefWnd.impl_->prefWndGeneralTab_.GetCompactBgPixmapFilename();
    preference.mixerDev_ = prefWnd.impl_->prefWndGeneralTab_.GetMixerDev();
#ifdef ENABLE_HTTP
    preference.convertUrlToCurrentEncoding_ = prefWnd.impl_->prefWndGeneralTab_.GetConvertUrl();
#endif

    preference.audioDriver_ = prefWnd.impl_->prefWndAudioDriverTab_.GetSelectedAudioDriver();
    preference.audioAlsa5Card_ = prefWnd.impl_->prefWndAudioDriverTab_.GetAlsa5Card();
    preference.audioAlsa5Dev_ = prefWnd.impl_->prefWndAudioDriverTab_.GetAlsa5Dev();
    preference.audioAlsa9Dev_ = prefWnd.impl_->prefWndAudioDriverTab_.GetAlsa9Dev();
    preference.audioOssDsp_ = prefWnd.impl_->prefWndAudioDriverTab_.GetOssDsp();
    preference.audioSunDev_ = prefWnd.impl_->prefWndAudioDriverTab_.GetSunDev();
    preference.audioAixsDev_ = prefWnd.impl_->prefWndAudioDriverTab_.GetAixsDev();

    preference.efxStereoEnhanceAmount_ = prefWnd.impl_->prefWndEffectorTab_.GetStereoEnhanceAmount();
    preference.efxRoomAmount_ = prefWnd.impl_->prefWndEffectorTab_.GetRoomAmount();
    preference.efxRoom2Amount_ = prefWnd.impl_->prefWndEffectorTab_.GetRoom2Amount();
    preference.efxHarmonyAmount_ = prefWnd.impl_->prefWndEffectorTab_.GetHarmonyAmount();
    preference.efxRhythmeAmount_ = prefWnd.impl_->prefWndEffectorTab_.GetRhythmeAmount();
    preference.efxDistortionAmount_ = prefWnd.impl_->prefWndEffectorTab_.GetDistortionAmount();
    preference.efxBassBoost_ = prefWnd.impl_->prefWndEffectorTab_.GetBassBoost();
    preference.efxBassBoost2_ = prefWnd.impl_->prefWndEffectorTab_.GetBassBoost2();
    preference.efxStereoEnhance_ = prefWnd.impl_->prefWndEffectorTab_.GetStereoEnhance();
    preference.efxTrebleBoost_ = prefWnd.impl_->prefWndEffectorTab_.GetTrebleBoost();
    preference.efxRoom_ = prefWnd.impl_->prefWndEffectorTab_.GetRoom();
    preference.efxRoom2_ = prefWnd.impl_->prefWndEffectorTab_.GetRoom2();
    preference.efxHarmony_ = prefWnd.impl_->prefWndEffectorTab_.GetHarmony();
    preference.efxRhythme_ = prefWnd.impl_->prefWndEffectorTab_.GetRhythme();
    preference.efxDistortion_ = prefWnd.impl_->prefWndEffectorTab_.GetDistortion();
    preference.efxGraphicEQ_ = prefWnd.impl_->prefWndGraphicEQTab_.GetGraphicEQ();

    preference.effect_ = prefWnd.impl_->prefWndEffectorTab_.GetEffector() || \
        prefWnd.impl_->prefWndGraphicEQTab_.GetGraphicEQ();

    preference.efxGraphicEQGain_[0] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ50();
    preference.efxGraphicEQGain_[1] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ100();
    preference.efxGraphicEQGain_[2] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ200();
    preference.efxGraphicEQGain_[3] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ400();
    preference.efxGraphicEQGain_[4] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ800();
    preference.efxGraphicEQGain_[5] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ1500();
    preference.efxGraphicEQGain_[6] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ3k();
    preference.efxGraphicEQGain_[7] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ5k();
    preference.efxGraphicEQGain_[8] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ7k();
    preference.efxGraphicEQGain_[9] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ10k();
    preference.efxGraphicEQGain_[10] = prefWnd.impl_->prefWndGraphicEQTab_.GetEQ15k();
}

/// @brief Output to ostream, generally file.
void operator<<(std::ostream& os, const PrefWnd::PrefWndImpl& prefWndImpl)
{
    const bool effect = prefWndImpl.prefWndEffectorTab_.GetEffector() || prefWndImpl.prefWndGraphicEQTab_.GetGraphicEQ();

    os << "GeneralTrayIconAction = " << prefWndImpl.prefWndGeneralTab_.GetTrayIconAction() << std::endl;
    os << "GeneralAssumeCharset = " << prefWndImpl.prefWndGeneralTab_.GetAssumeCharset() << std::endl;
    os << "GeneralCompactBgPixmap = " << prefWndImpl.prefWndGeneralTab_.GetCompactBgPixmapFilename() << std::endl;
#ifdef ENABLE_HTTP
    os << "GeneralConvertUrlToCurrentEncoding = " << prefWndImpl.prefWndGeneralTab_.GetConvertUrl() << std::endl;
#endif
    os << "AudioDriver = " << prefWndImpl.prefWndAudioDriverTab_.GetSelectedAudioDriver() << std::endl;
    os << "AudioAlsa5Card = " << prefWndImpl.prefWndAudioDriverTab_.GetAlsa5Card() << std::endl;
    os << "AudioAlsa5Dev = " << prefWndImpl.prefWndAudioDriverTab_.GetAlsa5Dev() << std::endl;
    os << "AudioAlsa9Dev = " << prefWndImpl.prefWndAudioDriverTab_.GetAlsa9Dev() << std::endl;
    os << "AudioOssDsp = " << prefWndImpl.prefWndAudioDriverTab_.GetOssDsp() << std::endl;
    os << "AudioSunDev = " << prefWndImpl.prefWndAudioDriverTab_.GetSunDev() << std::endl;
    os << "AudioAixsDev = " << prefWndImpl.prefWndAudioDriverTab_.GetAixsDev() << std::endl;
    os << "MixerDev = " << prefWndImpl.prefWndGeneralTab_.GetMixerDev() << std::endl;
    os << "Effect = " << effect << std::endl;			
    os << "EfxBassBoost = " << prefWndImpl.prefWndEffectorTab_.GetBassBoost() << std::endl;
    os << "EfxBassBoost2 = " << prefWndImpl.prefWndEffectorTab_.GetBassBoost2() << std::endl;
    os << "EfxTrebleBoost = " << prefWndImpl.prefWndEffectorTab_.GetTrebleBoost() << std::endl;
    os << "EfxStereoEnhance = " << prefWndImpl.prefWndEffectorTab_.GetStereoEnhance() << std::endl;
    os << "EfxStereoEnhanceAmount = " << prefWndImpl.prefWndEffectorTab_.GetStereoEnhanceAmount() << std::endl;
    os << "EfxRoom = " << prefWndImpl.prefWndEffectorTab_.GetRoom() << std::endl;
    os << "EfxRoom2 = " << prefWndImpl.prefWndEffectorTab_.GetRoom2() << std::endl;
    os << "EfxRoomAmount = " << prefWndImpl.prefWndEffectorTab_.GetRoomAmount() << std::endl;
    os << "EfxRoom2Amount = " << prefWndImpl.prefWndEffectorTab_.GetRoom2Amount() << std::endl;
    os << "EfxHarmony = " << prefWndImpl.prefWndEffectorTab_.GetHarmony() << std::endl;
    os << "EfxHarmonyAmount = " << prefWndImpl.prefWndEffectorTab_.GetHarmonyAmount() << std::endl;
    os << "EfxRhythme = " << prefWndImpl.prefWndEffectorTab_.GetRhythme() << std::endl;
    os << "EfxRhythmeAmount = " << prefWndImpl.prefWndEffectorTab_.GetRhythmeAmount() << std::endl;
    os << "EfxDistortion = " << prefWndImpl.prefWndEffectorTab_.GetDistortion() << std::endl;
    os << "EfxDistortionAmount = " << prefWndImpl.prefWndEffectorTab_.GetDistortionAmount() << std::endl;
    os << "EfxGraphicEQ = " << prefWndImpl.prefWndGraphicEQTab_.GetGraphicEQ() << std::endl;
    os << "EfxGraphicEQGain50 = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ50() << std::endl;
    os << "EfxGraphicEQGain100 = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ100() << std::endl;
    os << "EfxGraphicEQGain200 = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ200() << std::endl;
    os << "EfxGraphicEQGain400 = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ400() << std::endl;
    os << "EfxGraphicEQGain800 = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ800() << std::endl;
    os << "EfxGraphicEQGain1.5k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ1500() << std::endl;
    os << "EfxGraphicEQGain3k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ3k() << std::endl;
    os << "EfxGraphicEQGain5k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ5k() << std::endl;
    os << "EfxGraphicEQGain7k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ7k() << std::endl;
    os << "EfxGraphicEQGain10k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ10k() << std::endl;
    os << "EfxGraphicEQGain15k = " << prefWndImpl.prefWndGraphicEQTab_.GetEQ15k() << std::endl;
}
